package com.baijiayun.playback;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.playback.bean.models.imodels.IMessageModel;
import com.baijiayun.videoplayer.core.demo.R;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

/**
 * Created by yanglei on 2016/12/22.
 */

public class MessageListFragment extends Fragment {

    private PBRoom mRoom;
    private Disposable disposable;

    public void setRoom(PBRoom room) {
        mRoom = room;
        disposable = mRoom.getChatVM().getObservableOfNotifyDataChange()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMessageModels -> messageAdapter.notifyDataSetChanged());
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_user_list, container, false);
        RecyclerView mRecyclerView = view.findViewById(R.id.userList);
        mRecyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        mRecyclerView.setAdapter(messageAdapter);
        return view;
    }

    private class ViewHolder extends RecyclerView.ViewHolder {

        private TextView messageInfo;

        public ViewHolder(View itemView) {
            super(itemView);
            messageInfo = itemView.findViewById(R.id.userinfo);
        }
    }

    private RecyclerView.Adapter messageAdapter = new RecyclerView.Adapter<ViewHolder>() {

        @Override
        public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.layout_item_user, parent, false);
            return new ViewHolder(view);
        }

        @Override
        public void onBindViewHolder(ViewHolder holder, int position) {
            IMessageModel messageModel = mRoom.getChatVM().getMessage(position);

            holder.messageInfo.setText(messageModel.getFrom().getName() + "（group=" + messageModel.getFrom().getGroup() + ", type=" + messageModel.getFrom().getType().getType() + "）:" + messageModel.getContent());
        }

        @Override
        public int getItemCount() {
            return mRoom == null ? 0 : mRoom.getChatVM().getMessageCount();
        }
    };


    @Override
    public void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(disposable);
    }
}
