package com.baijiayun.playback;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import com.baijiayun.videoplayer.core.demo.R;

import java.util.Objects;

/**
 * 回放调试面板
 */
public class PBDebugFragment extends Fragment {

    private OnFragmentInteractionListener mListener;

    private EditText roomIdEt, tokenEt, sessionIdEt, seekTimeEt;
    private EditText messageGroupEt;
    private Button enterRoomBtn, playBtn, pauseBtn, seekBtn;
    private TextView launchStepTv, playerStatusTv, playerTimeTv, announcementTv;
    private CheckBox onlyTeacherAssistantMessageCheckbox;
    private SharedPreferences sharedPreferences;


    public PBDebugFragment() {
        // Required empty public constructor
    }

    public static PBDebugFragment newInstance(String param1, String param2) {
        PBDebugFragment fragment = new PBDebugFragment();
        Bundle args = new Bundle();
        args.putString("param1", param1);
        args.putString("param2", param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        sharedPreferences = Objects.requireNonNull(getActivity()).getSharedPreferences("demo_attr_cache", Context.MODE_PRIVATE);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_pbdebug, container, false);
        roomIdEt = view.findViewById(R.id.room_id_et);
        tokenEt = view.findViewById(R.id.token_et);
        sessionIdEt = view.findViewById(R.id.sessiod_et);
        messageGroupEt = view.findViewById(R.id.message_group_et);
        enterRoomBtn = view.findViewById(R.id.enter_room_btn);
        playBtn = view.findViewById(R.id.play_btn);
        pauseBtn = view.findViewById(R.id.pause_btn);
        seekBtn = view.findViewById(R.id.seek_btn);
        seekTimeEt = view.findViewById(R.id.et_seek_time);
        launchStepTv = view.findViewById(R.id.launch_step_tv);
        playerStatusTv = view.findViewById(R.id.tv_player_status);
        playerTimeTv = view.findViewById(R.id.tv_player_time);
        announcementTv = view.findViewById(R.id.announcement_tv);
        onlyTeacherAssistantMessageCheckbox = view.findViewById(R.id.message_checkbox);
        return view;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        roomIdEt.setText(sharedPreferences.getString("pb_room_id", ""));
        tokenEt.setText(sharedPreferences.getString("pb_token", "test12345678"));
        sessionIdEt.setText(sharedPreferences.getString("pb_session_id", "-1"));
        enterRoomBtn.setOnClickListener(v -> {
            if(!TextUtils.isEmpty(roomIdEt.getText().toString())){
                SharedPreferences.Editor editor = sharedPreferences.edit();
                editor.putString("pb_room_id", roomIdEt.getText().toString());
                editor.putString("pb_token", tokenEt.getText().toString());
                editor.putString("pb_session_id", sessionIdEt.getText().toString());
                editor.apply();
                mListener.enterRoomByFragment(roomIdEt.getText().toString(), tokenEt.getText().toString(), sessionIdEt.getText().toString());
            }
        });
        playBtn.setOnClickListener(v -> mListener.playByFragment());
        pauseBtn.setOnClickListener(v -> mListener.pauseByFragment());
        seekBtn.setOnClickListener(v -> mListener.seekByFragment(Integer.parseInt(seekTimeEt.getText().toString())));
        messageGroupEt.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                // do nothing
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                // do nothing
            }

            @Override
            public void afterTextChanged(Editable s) {
                int group = -1;
                String groupStr = s.toString();
                try {
                    if (!TextUtils.isEmpty(groupStr)) {
                        group = Integer.parseInt(groupStr);
                    }
                    mListener.onMessageGroupChange(group);
                } catch (Exception exception) {

                }

            }
        });
        onlyTeacherAssistantMessageCheckbox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            mListener.onTeacherAssistantMessageOnly(isChecked);
        });
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context
                    + " must implement OnFragmentInteractionListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    public void setLaunchStep(String tip) {
        launchStepTv.setText(tip);
    }

    public void setPlayerStatus(String status) {
        playerStatusTv.setText(status);
    }

    public void setPlayerTime(String time) {
        playerTimeTv.setText(time);
    }

    public void setAnnouncement(String announcement) {
        announcementTv.setVisibility(View.VISIBLE);
        announcementTv.setText(announcement);
    }

    public interface OnFragmentInteractionListener {

        void enterRoomByFragment(String roomIdOrVideoPath, String tokenOrSignalPath, String sessionId);

        void playByFragment();

        void pauseByFragment();

        void seekByFragment(int pos);

        void onMessageGroupChange(int group);

        void onTeacherAssistantMessageOnly(boolean taOnly);
    }
}
