package com.baijiayun.playback;

import android.Manifest;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.WindowManager;
import android.widget.Toast;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.viewpager.widget.PagerAdapter;
import androidx.viewpager.widget.ViewPager;
import com.baijiahulian.common.permission.AppPermissions;
import com.baijiayun.App;
import com.baijiayun.BJYPlayerSDK;
import com.baijiayun.download.DownloadModel;
import com.baijiayun.livebase.context.LPError;
import com.baijiayun.playback.mocklive.LPLaunchListener;
import com.baijiayun.playback.ppt.PPTView;
import com.baijiayun.videoplayer.IBJYVideoPlayer;
import com.baijiayun.videoplayer.VideoPlayerFactory;
import com.baijiayun.videoplayer.core.demo.R;
import com.baijiayun.videoplayer.widget.BJYPlayerView;
import com.google.android.material.tabs.TabLayout;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;

public class PlaybackActivity extends AppCompatActivity implements LPLaunchListener, PBDebugFragment.OnFragmentInteractionListener {
    private static final String TAG = "MainActivity";
    private PBRoom mRoom;
    private TabLayout mTabLayout;
    private BJYPlayerView playerView;
    private IBJYVideoPlayer bjyVideoPlayer;
    private ViewPager mViewPager;
    private PBDebugFragment debugFragment;
    private UserListFragment mUserListFragment;
    private MessageListFragment mMessageListFragment;
    //ppt控件
    private PPTView pptView;
    private long classId;
    private String classToken;
    private long sessionId;
    private int group = -1;
    private boolean taOnly;
    private DownloadModel videoDownloadModel, signalDownloadModel;
    private CompositeDisposable disposableContainer = new CompositeDisposable();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // 5.0+ 打开硬件加速
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED,
                WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED);
        if (savedInstanceState != null) {
            String FRAGMENTS_TAG = "android:support:fragments";
            savedInstanceState.remove(FRAGMENTS_TAG);
        }
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_playback);

        playerView = findViewById(R.id.playerView);
        mTabLayout = findViewById(R.id.tabLayout);
        mViewPager = findViewById(R.id.viewPager);

        pptView = findViewById(R.id.pptview);
        pptView.setBackgroundColor(ContextCompat.getColor(this, R.color.bjy_pb_ppt_bg));
        initViewPager();
        if (getIntent() != null) {
            videoDownloadModel = (DownloadModel) getIntent().getSerializableExtra("videoDownloadInfo");
            signalDownloadModel = (DownloadModel) getIntent().getSerializableExtra("signalDownloadInfo");
            if (videoDownloadModel != null && signalDownloadModel != null) {
                initRoom(false);
            }
        }
    }

    private void initRoom(boolean isOnLine) {
        if (TextUtils.isEmpty(classToken)) {
            classToken = "test12345678";
        }
        if (isOnLine) {
            mRoom = BJYPlayerSDK.newPlaybackRoom(this, classId, sessionId, classToken);
        } else {
            mRoom = BJYPlayerSDK.newPlaybackRoom(this, videoDownloadModel, signalDownloadModel);
        }
        Log.e("Enter", "enter room " + System.currentTimeMillis());

        bjyVideoPlayer = new VideoPlayerFactory.Builder()
                .setContext(this)
                .setLifecycle(getLifecycle())
                .setSupportBackgroundAudio(App.isPlayBackground)
                .setSupportLooping(App.isLoopPlay)
                .setSupportBreakPointPlay(App.isMemoryPlay)
                .build();
        bjyVideoPlayer.setUserInfo("点播回放SDK", "学员1");
        bjyVideoPlayer.bindPlayerView(playerView);
        bjyVideoPlayer.addOnPlayingTimeChangeListener((currentTime, duration) -> debugFragment.setPlayerTime("当前时间：" + currentTime + ", 总时间：" + duration));
        bjyVideoPlayer.addOnPlayerStatusChangeListener(status -> debugFragment.setPlayerStatus(status.name()));
        mRoom.bindPlayer(bjyVideoPlayer);

        disposableContainer.add(mRoom.getObservableOfAnnouncementChange().observeOn(AndroidSchedulers.mainThread())
                .subscribe(iAnnouncementModel -> debugFragment.setAnnouncement("公告:" + iAnnouncementModel.getContent())));

        if (isOnLine) {
            mRoom.enterRoom(PlaybackActivity.this);
        } else {
            AppPermissions.newPermissions(PlaybackActivity.this)
                    .request(Manifest.permission.READ_EXTERNAL_STORAGE,
                            Manifest.permission.WRITE_EXTERNAL_STORAGE)
                    .subscribe(aBoolean -> {
                        if (aBoolean) {
                            mRoom.enterRoom(PlaybackActivity.this);
                        } else {
                            Toast.makeText(PlaybackActivity.this, "无法操作", Toast.LENGTH_SHORT).show();
                        }
                    });
        }

        pptView.attachRoom(mRoom);
        mUserListFragment.setRoom(mRoom);
        mMessageListFragment.setRoom(mRoom);
    }

    private void initViewPager() {
        mViewPager.setAdapter(pagerAdapter);
        mViewPager.setOffscreenPageLimit(2);
        mTabLayout.setupWithViewPager(mViewPager);
        debugFragment = new PBDebugFragment();
        mUserListFragment = new UserListFragment();
        mMessageListFragment = new MessageListFragment();
    }

    @Override
    public void onLaunchSteps(final int step, final int totalStep) {
        debugFragment.setLaunchStep("enter Room " + (step * 100 / totalStep) + "%");
        Log.e("Enter", "enter Room " + (step * 100 / totalStep) + "% at:" + System.currentTimeMillis());
    }

    @Override
    public void onLaunchError(LPError error) {
        Toast.makeText(this, "进房间失败" + error.getCode() + " " + error.getMessage(), Toast.LENGTH_LONG).show();
    }

    @Override
    public void onLaunchSuccess(PBRoom room) {
        Log.e("Enter", "enter Room Success at:" + System.currentTimeMillis());
        mRoom.setMessageGroup(group);
        mRoom.setMessageModeTAOnly(taOnly);
    }

    private PagerAdapter pagerAdapter = new FragmentPagerAdapter(getSupportFragmentManager()) {
        @Override
        public Fragment getItem(int position) {
            switch (position) {
                case 0:
                    return debugFragment;
                case 1:
                    return mMessageListFragment;
                case 2:
                    return mUserListFragment;
                default:
                    return mMessageListFragment;
            }
        }

        @Override
        public CharSequence getPageTitle(int position) {
            switch (position) {
                case 0:
                    return "debug面板";
                case 1:
                    return "聊天";
                case 2:
                    return "用户名单";
                default:
                    return "聊天";
            }
        }

        @Override
        public int getCount() {
            return 3;
        }
    };


    /***
     ************  debug面板回调
     */
    @Override
    public void enterRoomByFragment(String roomIdOrVideoPath, String tokenOrSignalPath, String sessionId) {
        if (mRoom != null) {
            mRoom.quitRoom();
        }
        try {
            classId = Long.parseLong(roomIdOrVideoPath);
            this.sessionId = Long.parseLong(sessionId);
            this.classToken = tokenOrSignalPath;
        } catch (Exception e) {
            Toast.makeText(PlaybackActivity.this, "roomId格式错误：" + e.getMessage(), Toast.LENGTH_LONG).show();
        }
        initRoom(true);
    }

    @Override
    public void playByFragment() {
        if (bjyVideoPlayer != null && !bjyVideoPlayer.isPlaying()) {
            bjyVideoPlayer.play();
        }
    }

    @Override
    public void pauseByFragment() {
        if (bjyVideoPlayer != null) {
            bjyVideoPlayer.pause();
        }
    }

    @Override
    public void seekByFragment(int pos) {
        if (bjyVideoPlayer != null) {
            bjyVideoPlayer.seek(pos);
        }
    }

    @Override
    public void onMessageGroupChange(int group) {
        this.group = group;
        if (mRoom != null) {
            mRoom.setMessageGroup(group);
        }
    }

    @Override
    public void onTeacherAssistantMessageOnly(boolean taOnly) {
        this.taOnly = taOnly;
        if (mRoom != null) {
            mRoom.setMessageModeTAOnly(taOnly);
        }
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        pptView.destroy();
        if (mRoom != null) {
            mRoom.quitRoom();
        }
        disposableContainer.clear();
    }
}
